<?php

namespace Drupal\webform_jira_service_desk;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\webform\Plugin\WebformElementManagerInterface;
use Drupal\webform\Utility\WebformYaml;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform\WebformTokenManagerInterface;

/**
 * Integration with JIRA Service Desk REST API
 *
 * Allows the creation of requests in JIRA Service Desk.
 *
 * Some utility functions to provide human readable information on Service Desks,
 * Request Types, field names etc
 *
 * @package Drupal\webform_jira
 */
class JiraServiceDeskService {
  use StringTranslationTrait;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected $webformTokenManager;

  /**
   * @var \Drupal\webform\Plugin\WebformElementManagerInterface
   */
  protected $webformElementManager;

  /**
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * JiraServiceDeskService constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   * @param \Drupal\webform\WebformTokenManagerInterface $webform_token_manager
   * @param \Drupa\webform\WebformElementManagerInterface $webform_element_manager
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger_factory,
    WebformTokenManagerInterface $webform_token_manager,
    WebformElementManagerInterface $webform_element_manager,
    MessengerInterface $messenger
  ) {
    $this->logger = $logger_factory->get('webform_jira_service_desk');
    $this->webformTokenManager = $webform_token_manager;
    $this->webformElementManager = $webform_element_manager;
    $this->messenger = $messenger;
    $this->configureClient();
  }

  /**
   * Get list of Service Desks exposed in JIRA
   *
   * @return array
   *      of id => name pairs
   */
  public function getAvailableServiceDesks() {
    try {
      $response = $this->httpClient->request('GET', 'rest/servicedeskapi/servicedesk');
      $response_data = Json::decode($response->getBody());

      $desks = [];
      foreach ($response_data['values'] as $value) {
        $desks[$value['id']] = $value['projectName'];
      }
      return $desks;
    }
    catch (\GuzzleHttp\Exception\BadResponseException $e) {
      // assuming that Guzzle has logged the errors here
      return [];
    }

  }

  /**
   * Get Request Types from JIRA
   *
   * @param int|string $serviceDeskId
   *
   * @return array
   *      of id => name pairs
   */
  public function getAvailableRequestTypes($serviceDeskId) {
    try {
      $response = $this->httpClient->request('GET', 'rest/servicedeskapi/servicedesk/' . $serviceDeskId . '/requesttype');
      $response_data = Json::decode($response->getBody());

      $types = [];
      foreach ($response_data['values'] as $value) {
        $types[$value['id']] = $value['name'];
      }
      return $types;
    }
    catch (\GuzzleHttp\Exception\BadResponseException $e) {
      // assuming that Guzzle has logged the errors here
      return [];
    }

  }

  /**
   * Get field information for the request type
   *
   * @param int|string $serviceDeskId
   * @param int|string $requestTypeId
   *
   * @return array
   *    with id, display, type keys
   */
  public function getRequestTypeFields($serviceDeskId, $requestTypeId) {
    try {
      $response = $this->httpClient->request('GET', 'rest/servicedeskapi/servicedesk/' . $serviceDeskId . '/requesttype/' . $requestTypeId . '/field');
      $response_data = Json::decode($response->getBody());

      $fields = [];
      foreach ($response_data['requestTypeFields'] as $value) {
        $fields[] = [
          'id' => $value['fieldId'],
          'display' => $value['name'],
          'type' => $value['jiraSchema']['type'],
        ];
      }
      return $fields;
    }
    catch (\GuzzleHttp\Exception\BadResponseException $e) {
      // assuming that Guzzle has logged the errors here
      return [];
    }

  }

  /**
   * Formats and submits the webform submission to JIRA Service Desk
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *
   * @return \JiraRestApi\Issue\Issue|null
   */
  public function createRequestFromSubmission(WebformSubmissionInterface $webform_submission) {
    $webform = $webform_submission->getWebform();
    $fields = (array) WebformYaml::decode($webform->getThirdPartySetting(
      'webform_jira_service_desk',
      'fields'
    ));

    if ($fields) {
      $post_data = $post_data = [
        "serviceDeskId" => $webform->getThirdPartySetting('webform_jira_service_desk', 'servicedesk_id'),
        "requestTypeId" => $webform->getThirdPartySetting('webform_jira_service_desk', 'request_type_id'),
        "requestFieldValues" => $this->mapFormFieldsToPostData($webform_submission, $fields),
      ];

      try {
        $response = $this->httpClient->request('POST',
          'rest/servicedeskapi/request',
          ['json' => $post_data]
        );

        if ($response->getStatusCode() == 201) {
          $response_data = Json::decode($response->getBody());
          $this->logger->notice('Submission submitted to JIRA as @jira_id', [
            '@jira_id' => $response_data['issueKey'],
          ]);

          return $response_data;
        }
        else {
          $this->logger->warning("JIRA request not created: @reason", ['@reason' => $response->getBody()]);
        }
      }
      catch (\GuzzleHttp\Exception\RequestException $e) {
        $this->logger->error($e->getMessage());
        $this->messenger->addWarning("Something went wrong and your request has not been submitted. Please try again later.");
      }
    }
    else {
      $this->logger->warning("Webform @title has JIRA Service Desk enabled with no fields set", ['@title' => $webform->label()]);
    }

    return NULL;
  }

  /**
   * Maps Submission data to the format and names that JIRA is expecting
   *
   * @param \Drupal\webform\WebformSubmissionInterface $submission
   * @param array $fields
   *
   * @return array
   */
  protected function mapFormFieldsToPostData($submission, $fields) {
    $formatted = [];
    foreach ($fields as $field) {
      if ($value = $this->processFieldAndSubmissionDataToValue($field, $submission)) {
        $formatted[$field['field_id']] = $value;
      }
    }
    return $formatted;
  }

  /**
   * Converts an individual field to the correct data type and field name for JIRA
   *
   * @param $field
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *
   * @return array|null|string
   * @throws \Exception
   */
  protected function processFieldAndSubmissionDataToValue($field, WebformSubmissionInterface $webform_submission) {
    if (empty($field['field_type'])) {
      throw new \Exception($this->t('field_type not specified in field'));
    }

    if ($value = $this->getValueByValueType($field, $webform_submission)) {
      if ($field['field_type'] === 'string') {
        return $value;
      }
      if ($field['field_type'] === 'option') {
        return ['value' => $value];
      }
      if ($field['field_type'] === 'option_multiple') {
        $list = [];
        foreach ($value as $option) {
          $list[] = ['value' => $option];
        }
        return $list;
      }
      if ($field['field_type'] === 'number') {
        return floatval(
          preg_replace("/[^-0-9\.]/", "", $value)
        );
      }

      throw new \Exception($this->t('field_type not found', ['@field_type' => $field['field_type']]));
    }
  }

  /**
   * Gets the value of the field as the correct type
   *
   * Will process tokens as required.
   *
   * @param $field
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *
   * @return array|null|string
   */
  protected function getValueByValueType($field, WebformSubmissionInterface $webform_submission) {
    $value = NULL;
    $data = $webform_submission->getData();
    if (!empty($field['value_type'])) {
      $value_type = $field['value_type'];
      if ($value_type == 'webform_element' && !empty($data[$field['value']])) {
        $value = $data[$field['value']];
      }
      if ($value_type == 'text' && !empty($field['value'])) {
        $value = $this->webformTokenManager->replace(
          $field['value'], $webform_submission, [], ['clear' => TRUE]
        );
      }
    }

    return $value;
  }

  /**
   * Configure httpClient to use the settings supplied by JiraServiceDeskConfigurationForm
   *
   * @return void
   */
  protected function configureClient() {
    $config = \Drupal::config('webform_jira_service_desk.service_configuration');

    $base_url = $config->get('host');
    $key_name = $config->get('user_password_key');
    $auth = \Drupal::service('key.repository')
      ->getKey($key_name)
      ->getKeyValues();

    $this->httpClient = \Drupal::service('http_client_factory')
      ->fromOptions([
        'base_uri' => $base_url,
        'auth' => [$auth['username'], $auth['password']],
      ]);
  }
}
